@@ -1,21 +1,20 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2016, Florian Kirsch,
+// Copyright (C) 2002-2023, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // opencsg.h 
@@ -30,8 +29,8 @@
 
 /// OpenCSG version. The version number has been introduced with version 1.3.2.
 /// Format is Major|Minor|Patch, each of them being 4-bit wide.
-#define OPENCSG_VERSION 0x0141
-#define OPENCSG_VERSION_STRING "OpenCSG 1.4.1"
+#define OPENCSG_VERSION 0x0160
+#define OPENCSG_VERSION_STRING "OpenCSG 1.6.0"
 
 namespace OpenCSG {
 
@@ -184,24 +183,15 @@ namespace OpenCSG {
     ///   - AutomaticOffscreenType: Chooses internally depending on available
     ///                  OpenGL extensions. If graphics hardware supports different
     ///                  extensions, with most precedence ARB frame buffer objects
-    ///                  are used, then EXT frame buffer objects, then PBuffers.
-    ///   - FrameBufferObject: Uses frame buffer objects. This method does 
-    ///                  not require context switches on the graphics hardware
-    ///                  to change between offscreen and main frame buffer, so
-    ///                  in theory this method should be faster. Both ARB and
-    ///                  EXT frame buffer objects OpenGL extensions are supported
-    ///                  internally; ARB frame buffer objects are used if both 
-    ///                  are supported by the graphics hardware.
-    ///   - PBuffer: Uses PBuffers. This is the older offscreen type, which
-    ///                  is likely to work with older graphics hardware and
-    ///                  drivers.
+    ///                  are used, then EXT frame buffer objects.
+    ///   - FrameBufferObject: Same as AutomaticOffscreenType.
+    ///   - PBuffer (2): Not supported anymore
     ///   - FrameBufferObjectARB: Forces ARB frame buffer objects to be used.
     ///   - FrameBufferObjectEXT: Forces EXT frame buffer objects to be used.
     ///   - OffscreenTypeUnused: Invalid input.
     enum OffscreenType {
         AutomaticOffscreenType = 0,
         FrameBufferObject      = 1,
-        PBuffer                = 2,
         FrameBufferObjectARB   = 3,
         FrameBufferObjectEXT   = 4,
         OffscreenTypeUnused    = 5
@@ -229,11 +219,10 @@ namespace OpenCSG {
         OptimizationUnused    = 4
     };
 
-
     /// Setting the context is required for applications rendering with
     /// OpenCSG in different OpenGL windows with OpenGL contexts that
     /// are not shared. This is needed for internal OpenGL resources,
-    /// such as PBuffers or frame buffer objects, which are created in
+    /// such as frame buffer objects, which are created in
     /// in the render() method and recycled when render() is called again.
     /// The resources are only valid in the OpenGL context they have
     /// been created (and in contexts shared with this context).
@@ -251,19 +240,6 @@ namespace OpenCSG {
     /// context. 
     void freeResources();
 
-
-    /// Alternate render() function provided for compatibility with older
-    /// versions of OpenCSG (version 1.1.1 and before). The function performs
-    /// CSG rendering as the render() function described above. Additionally,
-    /// the Algorithm parameter must and the DepthComplexityAlgorithm parameter
-    /// can be provided. They supercede the options set by setOption().
-    /// This function is considered obsolete. New software should use the
-    /// render() function above and set the Options for rendering in separate
-    /// setOption() calls. 
-    void render(const std::vector<Primitive*>& primitives, 
-                Algorithm, 
-                DepthComplexityAlgorithm = NoDepthComplexitySampling);
-
 } // namespace OpenCSG
 
 #endif // __OpenCSG__opencsg_h__
