@@ -1,35 +1,28 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2016, Florian Kirsch,
+// Copyright (C) 2002-2023, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // channelManager.cpp
 //
 
 #include "opencsgConfig.h"
-#include <GL/glew.h>
-#ifdef _WIN32
-#include <GL/wglew.h>
-#elif !defined(__APPLE__)
-#include <GL/glxew.h>
-#endif
-
 #include "channelManager.h"
+
 #include "context.h"
 #include "offscreenBuffer.h"
 #include "openglHelper.h"
@@ -42,8 +35,6 @@ namespace OpenCSG {
 
     namespace {
 
-        GLint FaceOrientation = GL_CCW;
-
         int nextPow2(int value) {
             if(value <= 0) { return 0; }
             int result = 1;
@@ -71,7 +62,7 @@ namespace OpenCSG {
             int mSecondMax;
             int mCounter;
         public:
-            MaximumMemorizer() : mMax(0), mSecondMax(-1) { }
+            MaximumMemorizer() : mMax(0), mSecondMax(-1), mCounter(0) { }
             void newValue(int v) {
                 if (v>=mMax) {
                     mMax = v;
@@ -97,20 +88,23 @@ namespace OpenCSG {
 
     ChannelManager::ChannelManager()
       : mOffscreenBuffer(0)
+      , mInOffscreenBuffer(false)
+      , mCurrentChannel(NoChannel)
+      , mOccupiedChannels(NoChannel)
     {
         glPushAttrib(GL_ALL_ATTRIB_BITS);
         glDisable(GL_LIGHTING);
         glDisable(GL_TEXTURE_1D);
         glDisable(GL_TEXTURE_2D);
-        if (GLEW_ARB_texture_rectangle || GLEW_EXT_texture_rectangle || GLEW_NV_texture_rectangle)
+        if (   OPENCSG_HAS_EXT(ARB_texture_rectangle)
+            || OPENCSG_HAS_EXT(EXT_texture_rectangle)
+            || OPENCSG_HAS_EXT(NV_texture_rectangle))
             glDisable(GL_TEXTURE_RECTANGLE_ARB);
         glDisable(GL_TEXTURE_3D); // OpenGL 1.2 - take this as given
-        if (GL_ARB_texture_cube_map)
+        if (OPENCSG_HAS_EXT(ARB_texture_cube_map))
             glDisable(GL_TEXTURE_CUBE_MAP_ARB);
         glDisable(GL_BLEND);
 
-        glGetIntegerv(GL_FRONT_FACE, &FaceOrientation);
-
         glGetFloatv(GL_MODELVIEW_MATRIX, OpenGL::modelview);
         glGetFloatv(GL_PROJECTION_MATRIX, OpenGL::projection);
         glGetIntegerv(GL_VIEWPORT, OpenGL::canvasPos);
@@ -137,33 +131,15 @@ namespace OpenCSG {
         if (   newOffscreenType == OpenCSG::AutomaticOffscreenType
             || newOffscreenType == OpenCSG::FrameBufferObject
         ) {
-            if (GLEW_ARB_framebuffer_object) {
+            if (OPENCSG_HAS_EXT(ARB_framebuffer_object)) {
                 newOffscreenType = OpenCSG::FrameBufferObjectARB;
             }
             else
-            if (   GLEW_EXT_framebuffer_object
-                && GLEW_EXT_packed_depth_stencil
+            if (   OPENCSG_HAS_EXT(EXT_framebuffer_object)
+                && OPENCSG_HAS_EXT(EXT_packed_depth_stencil)
             ) {
                 newOffscreenType = OpenCSG::FrameBufferObjectEXT;
             }
-            else
-            if (newOffscreenType == OpenCSG::AutomaticOffscreenType
-#ifndef OPENCSG_HAVE_PBUFFER
-                && false
-#else
-#ifdef WIN32
-                && WGLEW_ARB_pbuffer
-                && WGLEW_ARB_pixel_format
-#elif !defined(__APPLE__)
-                && GLXEW_SGIX_pbuffer
-                && GLXEW_SGIX_fbconfig
-#else
-                && false
-#endif
-#endif // OPENCSG_HAVE_PBUFFER
-            ) {
-                newOffscreenType = OpenCSG::PBuffer;
-            }
             else {
                 // At least one set of the above OpenGL extensions is required
                 return false;
@@ -172,27 +148,31 @@ namespace OpenCSG {
 
         mOffscreenBuffer = OpenGL::getOffscreenBuffer(newOffscreenType);
 
-        if (!mOffscreenBuffer) {
+        if (!mOffscreenBuffer)
+        {
             // Creating the offscreen buffer failed, maybe the OpenGL extension
             // for the specific offscreen buffer type is not supported
             return false;
         }
 
+        if (!mOffscreenBuffer->ReadCurrent())
+        {
+            return false;
+        }
+
         const int dx = OpenGL::canvasPos[2] - OpenGL::canvasPos[0];
         const int dy = OpenGL::canvasPos[3] - OpenGL::canvasPos[1];
 
         int tx = dx;
         int ty = dy;
         // We don't need to enlarge the texture to the next largest power-of-two size if:
-        // - any of the texture rectangle extensions is supported
-        //   (texture rectangle is no problem for FBO; for pbuffers we fallback to copy-to-texture
-        //    if the required WGL-extensions for texture rectangle are missing - always on Linux)
-        // - Otherwise for FBO, if we have the GLEW_ARB_texture_non_power_of_two extension
+        // - the ARB_texture_non_power_of_two extension is supported
+        // - or any of the texture rectangle extensions is supported
         // Negating this gives the following expression from hell:
-        if (   !GLEW_ARB_texture_rectangle
-            && !GLEW_EXT_texture_rectangle
-            && !GLEW_NV_texture_rectangle
-            && (newOffscreenType == OpenCSG::PBuffer || !GLEW_ARB_texture_non_power_of_two)
+        if (   !OPENCSG_HAS_EXT(ARB_texture_non_power_of_two)
+            && !OPENCSG_HAS_EXT(ARB_texture_rectangle)
+            && !OPENCSG_HAS_EXT(EXT_texture_rectangle)
+            && !OPENCSG_HAS_EXT(NV_texture_rectangle)
         ) {
             // blow up the texture to legal power-of-two size :-(
             tx = nextPow2(dx);
@@ -221,8 +201,9 @@ namespace OpenCSG {
 
         bool rebuild = false;
 
-        if (!mOffscreenBuffer->IsInitialized()) {
-            if (!mOffscreenBuffer->Initialize(sizeX.getMax(), sizeY.getMax(), true, false)) {
+        if (!mOffscreenBuffer->IsInitialized())
+        {
+            if (!mOffscreenBuffer->Initialize(sizeX.getMax(), sizeY.getMax())) {
                 // Initializing the offscreen buffer failed, maybe the OpenGL extension
                 // for the specific offscreen buffer type is not supported
                 return false;
@@ -230,18 +211,18 @@ namespace OpenCSG {
             rebuild = true;
         }
         // tx == ty == 0 happens if the window is minimized, in this case don't touch a thing
-        else if (tx != 0 && ty != 0) {
-            if (   mOffscreenBuffer->GetWidth() != sizeX.getMax()
-                || mOffscreenBuffer->GetHeight() != sizeY.getMax()
-            ) {
-                if (!mOffscreenBuffer->Resize(sizeX.getMax(), sizeY.getMax())) {
-                    // Resizing the offscreen buffer failed, maybe the OpenGL extension
-                    // for the specific offscreen buffer type is not supported. More 
-                    // likely this is a programming error in Resize(). 
-                    return false;
-                }
-                rebuild = true;
+        else if (tx != 0 && ty != 0 &&
+                    (   mOffscreenBuffer->GetWidth() != sizeX.getMax()
+                     || mOffscreenBuffer->GetHeight() != sizeY.getMax()
+                )   )
+        {
+            if (!mOffscreenBuffer->Resize(sizeX.getMax(), sizeY.getMax())) {
+                // Resizing the offscreen buffer failed, maybe the OpenGL extension
+                // for the specific offscreen buffer type is not supported. More
+                // likely this is a programming error in Resize().
+                return false;
             }
+            rebuild = true;
         }
 
         if (rebuild) {
@@ -277,7 +258,7 @@ namespace OpenCSG {
         // find free channel
         if ((mOccupiedChannels & Alpha) == 0) {
             channel = Alpha;
-        }  else if (GLEW_ARB_texture_env_dot3) {
+        }  else if (OPENCSG_HAS_EXT(ARB_texture_env_dot3)) {
             if ((mOccupiedChannels & Red) == 0)   {
                 channel = Red;
             } else if ((mOccupiedChannels & Green) == 0) {
@@ -293,9 +274,6 @@ namespace OpenCSG {
     Channel ChannelManager::request() {
         if (!mInOffscreenBuffer) {
             mOffscreenBuffer->BeginCapture();
-            if (mOffscreenBuffer->haveSeparateContext()) {
-                glFrontFace(FaceOrientation);
-            }
 
             mInOffscreenBuffer = true;
 
@@ -303,14 +281,6 @@ namespace OpenCSG {
             mOccupiedChannels = NoChannel;
         }
 
-        if (mOffscreenBuffer->haveSeparateContext()) {
-            glViewport(OpenGL::canvasPos[0], OpenGL::canvasPos[1], OpenGL::canvasPos[2], OpenGL::canvasPos[3]);
-            glMatrixMode(GL_PROJECTION);
-            glLoadMatrixf(OpenGL::projection);
-            glMatrixMode(GL_MODELVIEW);
-            glLoadMatrixf(OpenGL::modelview);
-        }
-
         mCurrentChannel = find();
         mOccupiedChannels |= mCurrentChannel;
         return mCurrentChannel;
@@ -416,13 +386,8 @@ namespace OpenCSG {
         float factorX = static_cast<float>(dx);
         float factorY = static_cast<float>(dy);
 
-        // RenderTexture has a slight flaw: if you request a power-of-two texture
-        // by coincidence even though you expected a NPOT texture, you will not
-        // automatically notice that. That's for a bug that only happened at
-        // 512x512 canvas resolution. 
-        // Therefore we do not check for the extension, but simply for the texture format
-        // Update (08.04.2004): Fixed the flaw, but kept checking the texture format.
-        // Actually that seems safer, since it should work always
+        // Do not check for the non-power-of-two extension, but simply for
+        // the texture format. This seems safer, since it should work always.
         if (!isRectangularTexture()) {
             // with ordinary pow-of-two texture coordinates are between 0 and 1
             // but we must assure only the used part of the texture is taken.
@@ -452,7 +417,7 @@ namespace OpenCSG {
 
     void ChannelManager::resetProjectiveTexture(bool fixedFunction)
     {
-        if (fixedFunction && !mOffscreenBuffer->haveSeparateContext())
+        if (fixedFunction)
         {
             glDisable(GL_TEXTURE_GEN_S);
             glDisable(GL_TEXTURE_GEN_T);
@@ -473,7 +438,7 @@ namespace OpenCSG {
             glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
         } else {
             // replicate color into alpha
-            if (GLEW_ARB_texture_env_dot3) {
+            if (OPENCSG_HAS_EXT(ARB_texture_env_dot3)) {
                 switch (channel) {
                 case Red: 
                     glColor3f(1.0f, 0.5f, 0.5f); 
@@ -493,12 +458,12 @@ namespace OpenCSG {
                 assert(0);
             }
 
-            glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_ARB);
-            glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_ARB, GL_DOT3_RGBA_ARB);
-            glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_ARB, GL_TEXTURE);
-            glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_ARB, GL_SRC_COLOR);
-            glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB_EXT, GL_PRIMARY_COLOR);
-            glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB_EXT, GL_SRC_COLOR);
+            glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
+            glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_DOT3_RGBA);
+            glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE);
+            glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);
+            glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PRIMARY_COLOR);
+            glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR);
         }
     }
 
