@@ -1,20 +1,19 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2006-2016, Florian Kirsch
+// Copyright (C) 2006-2023, Florian Kirsch
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // frameBufferObject.h 
@@ -27,7 +26,7 @@
 
 #include "opencsgConfig.h"
 #include "offscreenBuffer.h"
-#include <GL/glew.h>
+#include "openglExt.h"
 
 namespace OpenCSG {
 
@@ -39,10 +38,12 @@ namespace OpenCSG {
             FrameBufferObject();
             virtual ~FrameBufferObject();
 
-            /// Call this once before use. shareObjects and copyContext are
-            /// ignored, since there are no separate contexts for frame
-            /// buffer objects anyway.
-            virtual bool Initialize(int width, int height, bool shareObjects=true, bool copyContext=false);
+            /// Reads the currently bound FBO.
+            virtual bool ReadCurrent();
+
+            /// Initializes the frame buffer object with the intended width and height.
+            /// The frame buffer object is created with RGBA and combined depth/stencil buffer.
+            virtual bool Initialize(int width, int height);
 
             /// checks whether Initialize has been called before or not
             virtual bool IsInitialized() const { return initialized; }
@@ -72,9 +73,6 @@ namespace OpenCSG {
             /// Returns the width of the frame buffer object.
             virtual int GetHeight() const { return height; }
 
-            /// Frame buffer objects do not change the OpenGL context.
-            virtual bool haveSeparateContext() const { return false; }
-
         protected:
             int          width;     // width of the frame buffer object
             int          height;    // height of the frame buffer object
@@ -88,6 +86,11 @@ namespace OpenCSG {
             int          oldFramebufferID;
 
             bool         initialized;
+
+        private:
+            FrameBufferObject(const FrameBufferObject&);
+            FrameBufferObject& operator=(const FrameBufferObject&);
+
         };
 
     } // namespace OpenGL
