@@ -1,20 +1,19 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2006-2016, Florian Kirsch
+// Copyright (C) 2006-2023, Florian Kirsch
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // offscreenBuffer.h 
@@ -38,12 +37,14 @@ namespace OpenCSG {
 
             virtual ~OffscreenBuffer() {};
 
-            /// Call this once before use.  Set bShare to true to share lists, textures, 
-            /// and program objects between the render texture context and the 
-            /// current active GL context.
-            virtual bool Initialize(int width, int height, 
-                                    bool shareObjects=true, 
-                                    bool copyContext=false) = 0;
+            /// Reads the current settings, such as the currently bound FBO.
+            /// This should be called before every use of the offscreen buffer
+            /// even if Initialize() or Resize() do not need to be called.
+            virtual bool ReadCurrent() = 0;
+
+            /// Call this once before use, to initialize the offscreen buffer
+            /// with the intended width and height.
+            virtual bool Initialize(int width, int height) = 0;
 
             /// checks whether Initialize has been called before or not
             virtual bool IsInitialized() const = 0;
@@ -73,10 +74,6 @@ namespace OpenCSG {
             virtual int GetWidth() const = 0;
             /// Returns the width of the offscreen buffer.
             virtual int GetHeight() const = 0;
-
-            /// Returns whether the offscreen buffer has a rendering context different from
-            /// the rendering context of the caller or not.
-            virtual bool haveSeparateContext() const = 0;
         };
     } // namespace OpenGL
 
