@@ -1,21 +1,20 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2014, Florian Kirsch,
+// Copyright (C) 2002-2023, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // renderGoldfeather.cpp
@@ -32,7 +31,6 @@
 #include "openglHelper.h"
 #include "primitiveHelper.h"
 #include "scissorMemo.h"
-#include "stencilManager.h"
 #include <algorithm>
 
 namespace OpenCSG {
@@ -40,7 +38,6 @@ namespace OpenCSG {
     namespace {
 
         ScissorMemo* scissor;
-        OpenGL::StencilManager* stencilMgr;
 
         class GoldfeatherChannelManager : public ChannelManagerForBatches {
         public:
@@ -49,13 +46,15 @@ namespace OpenCSG {
 
         void GoldfeatherChannelManager::merge() {
 
-            setupProjectiveTexture();
+            bool isFixedFunction = true;
+            setupProjectiveTexture(isFixedFunction);
 
             glEnable(GL_ALPHA_TEST);
             glAlphaFunc(GL_GEQUAL, 0.5f); // accuracy issue with GL_EQUAL 1.0 on FX5600
             glEnable(GL_DEPTH_TEST);
             glDepthFunc(GL_LESS);
             glDepthMask(GL_TRUE);
+            glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
 
             std::vector<Channel> channels = occupied();
             for (std::vector<Channel>::const_iterator c = channels.begin(); c!=channels.end(); ++c) {
@@ -78,7 +77,8 @@ namespace OpenCSG {
                     // shapes of interest: we need to determine the appropriate layer of 
                     // the shapes, using stencil counting
                     glClearStencil(0);
-                    stencilMgr->clear();
+                    glStencilMask(OpenGL::stencilMask);
+                    glClear(GL_STENCIL_BUFFER_BIT);
                     OpenGL::renderLayer(getLayer(*c), primitives);
                     glDisable(GL_STENCIL_TEST);
                 }
@@ -90,7 +90,7 @@ namespace OpenCSG {
 
             scissor->disableScissor();
 
-            resetProjectiveTexture();
+            resetProjectiveTexture(isFixedFunction);
 
             clear();
         }
@@ -279,7 +279,6 @@ namespace OpenCSG {
         Batcher batches(primitives);
 
         scissor->setIntersected(primitives);
-        stencilMgr = OpenGL::getStencilManager(scissor->getIntersectedArea());
 
         for (std::vector<Batch>::const_iterator itr = batches.begin(); itr != batches.end(); ++itr) {
             unsigned int maxConvexity = Algo::getConvexity(*itr);
@@ -334,10 +333,8 @@ namespace OpenCSG {
         }
 
         channelMgr->free();
-        stencilMgr->restore();
-        
+
         delete scissor;
-        delete stencilMgr;
     }
 
     bool renderOcclusionQueryGoldfeather(const std::vector<Primitive*>& primitives)
@@ -347,7 +344,6 @@ namespace OpenCSG {
         unsigned int layer = 0;
 
         scissor->setIntersected(primitives);
-        stencilMgr = OpenGL::getStencilManager(scissor->getIntersectedArea());
         scissor->setCurrent(primitives);
 
         OpenGL::OcclusionQuery* occlusionTest = 0;
@@ -400,15 +396,21 @@ namespace OpenCSG {
             scissor->disableScissor();
 
             ++layer;
+
+            // There are more layers than the stencil buffer allows us to
+            // render. Return claiming success (which is cheating, since
+            // we were not really successful)
+            if (layer == OpenGL::stencilMax) {
+                retVal = true;
+                break;
+            }
         }
 
         delete occlusionTest;
 
         channelMgr->free();
-        stencilMgr->restore();
 
         delete scissor;
-        delete stencilMgr;
 
         return retVal;
     }
@@ -418,11 +420,9 @@ namespace OpenCSG {
         scissor = new ScissorMemo;
 
         scissor->setIntersected(primitives);
-        stencilMgr = OpenGL::getStencilManager(scissor->getIntersectedArea());
         scissor->setCurrent(primitives);
         scissor->enableScissor();
 
-        stencilMgr->clear();
         unsigned int depthComplexity = OpenGL::calcMaxDepthComplexity(primitives, scissor->getIntersectedArea());
 
         scissor->disableScissor();
@@ -456,10 +456,8 @@ namespace OpenCSG {
         }
 
         channelMgr->free();
-        stencilMgr->restore();
 
         delete scissor;
-        delete stencilMgr;
     }
 
     void renderGoldfeather(const std::vector<Primitive*>& primitives, DepthComplexityAlgorithm algorithm)
@@ -471,7 +469,9 @@ namespace OpenCSG {
             case OcclusionQuery:
                 if (renderOcclusionQueryGoldfeather(primitives))
                     break;  // success
-                            // else fall through (should not happen in practice due to the check for extensions of the caller)
+                // coming here should not happen in practice due to the check
+                // for extensions performed by the caller. Anyway, if it happens,
+                // fall through
             case NoDepthComplexitySampling:
                 renderGoldfeather(primitives);
                 break;
