@@ -1,21 +1,20 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2014, Florian Kirsch,
+// Copyright (C) 2002-2023, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // renderSCS.cpp
@@ -28,10 +27,13 @@
 #include "opencsgRender.h"
 #include "batch.h"
 #include "channelManager.h"
+#include "context.h"
 #include "occlusionQuery.h"
 #include "openglHelper.h"
 #include "primitiveHelper.h"
 #include "scissorMemo.h"
+
+#include <algorithm>
 #include <map>
 
 namespace OpenCSG {
@@ -40,8 +42,18 @@ namespace OpenCSG {
 
         ScissorMemo* scissor;
 
+        struct IdBufferId {
+            GLubyte r;
+            GLubyte g;
+            GLubyte b;
+            GLubyte a;
+            GLubyte* vec() {
+                return &r;
+            }
+        };
+
         struct RenderData {
-            unsigned int stencilID;
+            IdBufferId bufferId;
         };
 
         std::map<Primitive*, RenderData> gRenderInfo;
@@ -50,21 +62,24 @@ namespace OpenCSG {
             RenderData* dta = &(gRenderInfo.find(primitive))->second;
             return dta;
         }
-    
-        class SCSChannelManager : public ChannelManagerForBatches {
+
+        // Stores Ids in the alpha buffer only -> only 255 primitives are possible
+        class SCSChannelManagerAlphaOnly : public ChannelManagerForBatches {
         public:
             virtual void merge();
         };
 
-        void SCSChannelManager::merge() {
+        void SCSChannelManagerAlphaOnly::merge() {
 
-            setupProjectiveTexture();
+            bool isFixedFunction = true;
+            setupProjectiveTexture(isFixedFunction);
 
             glEnable(GL_ALPHA_TEST);
             glEnable(GL_CULL_FACE);
             glEnable(GL_DEPTH_TEST);
             glDepthFunc(GL_LESS);
             glDepthMask(GL_TRUE);
+            glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
 
             std::vector<Channel> channels = occupied();
             for (std::vector<Channel>::const_iterator c = channels.begin(); c!=channels.end(); ++c) {
@@ -77,7 +92,7 @@ namespace OpenCSG {
                 for (std::vector<Primitive*>::const_iterator j = primitives.begin(); j != primitives.end(); ++j) {
                     glCullFace((*j)->getOperation() == Intersection ? GL_BACK : GL_FRONT);
                     RenderData* primitiveData = getRenderData(*j);
-                    unsigned char id = primitiveData->stencilID;
+                    GLubyte id = primitiveData->bufferId.a;
 
                     // Here is an interesting bug, which happened on an ATI HD4670, but actually
                     // might happen on every hardware. I am not sure whether it can be solved 
@@ -99,6 +114,8 @@ namespace OpenCSG {
                     // lookup table containing the correct alpha reference values is 
                     // not required. However a bad feeling remains.
 
+                    // The SCSChannelManagerFragmentProgram path implemented below should fix this.
+
                     double alpha = static_cast<double>(id) / 255.0;
                     GLfloat fAlpha = static_cast<float>(alpha);
                     glAlphaFunc(GL_EQUAL, fAlpha);
@@ -113,25 +130,178 @@ namespace OpenCSG {
             glDisable(GL_CULL_FACE);
             glDepthFunc(GL_LEQUAL);
 
-            resetProjectiveTexture();
+            resetProjectiveTexture(isFixedFunction);
 
             clear();
         }
 
-        class IDGenerator {
+        // Stores Ids in the all components of the color buffer
+        // -> in theory, 2^32-1 primitives are possible
+        class SCSChannelManagerFragmentProgram : public ChannelManagerForBatches {
         public:
-            IDGenerator() { currentID = 0; };
-            unsigned int newID() { return ++currentID; };
-
-        private:
-            unsigned int currentID;
+            virtual Channel request();
+            virtual void merge();
         };
 
-        SCSChannelManager* channelMgr;
+        Channel SCSChannelManagerFragmentProgram::request() {
+            ChannelManagerForBatches::request();
+            mCurrentChannel = AllChannels;
+            mOccupiedChannels = mCurrentChannel;
+            return mCurrentChannel;
+        }
+
+        // Emulates eye texgen
+        static const char mergeVertexProgram[] =
+"!!ARBvp1.0 OPTION ARB_position_invariant;\n"
+"ATTRIB  pos = vertex.position;\n"
+"ATTRIB  col = vertex.color;\n"
+"OUTPUT  outCol = result.color;\n"
+"OUTPUT  outTex0 = result.texcoord[0];\n"
+"PARAM   mvpmat[4] = { state.matrix.mvp };\n"
+"PARAM   texmat[4] = { state.matrix.texture[0] };\n"
+"TEMP    eye;\n"
+"TEMP    tex;\n"
+"DP4     eye.x, mvpmat[0], pos;\n"
+"DP4     eye.y, mvpmat[1], pos;\n"
+"DP4     eye.z, mvpmat[2], pos;\n"
+"DP4     eye.w, mvpmat[3], pos;\n"
+"DP4     tex.x, texmat[0], eye;\n"
+"DP4     tex.y, texmat[1], eye;\n"
+"DP4     tex.z, texmat[2], eye;\n"
+"DP4     tex.w, texmat[3], eye;\n"
+"MOV     outTex0, tex;\n"
+"MOV     outCol, col;\n"
+"END";
+
+        // Subtract color from texture value, takes the absolute value
+        // and adds all components into each channel of the result, scaled by 2.0f.
+        // This way, all 32-bits of the color channel can be used
+        // for an 'equal' alpha test, i.e, to check if value in texture
+        // and color are equal.
+
+        // Note that 1.0f/255.0f cannot be the result of the above computation.
+        // Either the result is 0 (if all components were equal, disregarding
+        // numerical errors), or larger/equal than 2.0f/255.0f. The alpha function
+        // then is actually set to  GL_LESS, 1.0f/255.0f. This is robust,
+        // in contract to the GL_EQUAL in SCSChannelManagerAlphaOnly above.
+        // The scaling by 2.0f is required for NVidia hardware, which considers
+        // the alpha function GL_LESS, 0.5f/255.0f as GL_LESS, 0.0f for some reason.
+        static const char mergeFragmentProgramRect[] =
+"!!ARBfp1.0\n"
+"TEMP    temp;\n"
+"ATTRIB  tex0 = fragment.texcoord[0];\n"
+"ATTRIB  col0 = fragment.color;\n"
+"PARAM   scaleByTwo = { 2.0, 2.0, 2.0, 2.0 };\n"
+"OUTPUT  out = result.color;\n"
+"TXP     temp, tex0, texture[0], RECT;\n"
+"SUB     temp, temp, col0;\n"
+"ABS     temp, temp;\n"
+"DP4     out, temp, scaleByTwo;\n"
+"END";
+
+        static const char mergeFragmentProgram2D[] =
+"!!ARBfp1.0\n"
+"TEMP    temp;\n"
+"ATTRIB  tex0 = fragment.texcoord[0];\n"
+"ATTRIB  col0 = fragment.color;\n"
+"PARAM   scaleByTwo = { 2.0, 2.0, 2.0, 2.0 };\n"
+"OUTPUT  out = result.color;\n"
+"TXP     temp, tex0, texture[0], 2D;\n"
+"SUB     temp, temp, col0;\n"
+"ABS     temp, temp;\n"
+"DP4     out, temp, scaleByTwo;\n"
+"END";
+
+        void SCSChannelManagerFragmentProgram::merge()
+        {
+            GLuint vId = OpenGL::getARBVertexProgram(mergeVertexProgram, (sizeof(mergeVertexProgram) / sizeof(mergeVertexProgram[0])) - 1);
+            glBindProgramARB(GL_VERTEX_PROGRAM_ARB, vId);
+            glEnable(GL_VERTEX_PROGRAM_ARB);
+
+            GLuint fId =
+                 isRectangularTexture()
+                   ? OpenGL::getARBFragmentProgram(mergeFragmentProgramRect, (sizeof(mergeFragmentProgramRect) / sizeof(mergeFragmentProgramRect[0])) - 1)
+                   : OpenGL::getARBFragmentProgram(mergeFragmentProgram2D, (sizeof(mergeFragmentProgram2D) / sizeof(mergeFragmentProgram2D[0])) - 1);
+            glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, fId);
+            glEnable(GL_FRAGMENT_PROGRAM_ARB);
+
+            bool isFixedFunction = false;
+            setupProjectiveTexture(isFixedFunction);
+
+            glEnable(GL_ALPHA_TEST);
+            glEnable(GL_CULL_FACE);
+            glEnable(GL_DEPTH_TEST);
+            glDepthFunc(GL_LESS);
+            glDepthMask(GL_TRUE);
+            glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
+
+            glAlphaFunc(GL_LESS, 1.0f / 255.0f);
+
+            std::vector<Channel> channels;
+            channels.push_back(AllChannels);
+            for (std::vector<Channel>::const_iterator c = channels.begin(); c!=channels.end(); ++c) {
+
+                scissor->recall(*c);
+                scissor->enableScissor();
+
+                const std::vector<Primitive*> primitives = getPrimitives(*c);
+                for (std::vector<Primitive*>::const_iterator j = primitives.begin(); j != primitives.end(); ++j) {
+                    glCullFace((*j)->getOperation() == Intersection ? GL_BACK : GL_FRONT);
+                    RenderData* primitiveData = getRenderData(*j);
+                    GLubyte * id = primitiveData->bufferId.vec();
+                    glColor4ubv(id);
+                    (*j)->render();
+                }
+            }
+
+            scissor->disableScissor();
+
+            glDisable(GL_ALPHA_TEST);
+            glDisable(GL_CULL_FACE);
+            glDepthFunc(GL_LEQUAL);
+            glDisable(GL_FRAGMENT_PROGRAM_ARB);
+            glDisable(GL_VERTEX_PROGRAM_ARB);
+
+            resetProjectiveTexture(isFixedFunction);
+
+            clear();
+        }
+
+
+        ChannelManagerForBatches* getChannelManager() {
+
+            if (   OPENCSG_HAS_EXT(ARB_vertex_program)
+                && OPENCSG_HAS_EXT(ARB_fragment_program)
+            ) {
+                return new SCSChannelManagerFragmentProgram;
+            }
+
+            return new SCSChannelManagerAlphaOnly;
+        }
+
+
+        class IDGenerator {
+        public:
+            IDGenerator() : counter(0) {};
+            IdBufferId newID() {
+                ++counter;
+                IdBufferId newId;
+                newId.r =  (counter >> 24) & 0xff;
+                newId.g =  (counter >> 16) & 0xff;
+                newId.b =  (counter >>  8) & 0xff;
+                newId.a =  (counter >>  0) & 0xff;
+                return newId;
+            };
+
+        private:
+            unsigned int counter;
+        };
+
+        ChannelManagerForBatches* channelMgr;
 
         void renderIntersectedFront(const std::vector<Primitive*>& primitives) {
 
-            const int numberOfPrimitives = primitives.size();
+            const std::size_t numberOfPrimitives = primitives.size();
 
             glDepthMask(GL_TRUE);
 
@@ -142,7 +312,8 @@ namespace OpenCSG {
                 glCullFace(GL_BACK);
                 glEnable(GL_CULL_FACE);
                 RenderData* primitiveData = getRenderData(primitives[0]);
-                GLubyte b = primitiveData->stencilID; glColor4ub(b, b, b, b);
+                GLubyte * id = primitiveData->bufferId.vec();
+                glColor4ubv(id);
                 primitives[0]->render();
                 glDisable(GL_CULL_FACE);
                 glDepthFunc(GL_LESS);
@@ -160,7 +331,8 @@ namespace OpenCSG {
             {
                 for (std::vector<Primitive*>::const_iterator i = primitives.begin(); i != primitives.end(); ++i) {
                     RenderData* primitiveData = getRenderData(*i);
-                    GLubyte b = primitiveData->stencilID; glColor4ub(b, b, b, b);
+                    GLubyte * id = primitiveData->bufferId.vec();
+                    glColor4ubv(id);
                     (*i)->render();
                 }
             }
@@ -182,7 +354,7 @@ namespace OpenCSG {
             // where #back faces behind furthest front face != #intersected shapes
             // ->reset fragment
             channelMgr->renderToChannel(true);
-            glStencilFunc(GL_NOTEQUAL, numberOfPrimitives, OpenGL::stencilMask);
+            glStencilFunc(GL_NOTEQUAL, static_cast<GLint>(numberOfPrimitives), OpenGL::stencilMask);
             glDepthFunc(GL_ALWAYS);
             glDepthRange(0.0, 0.0);
             glDepthMask(GL_TRUE);
@@ -209,7 +381,7 @@ namespace OpenCSG {
             glEnable(GL_STENCIL_TEST);
             glEnable(GL_CULL_FACE);
 
-            int stencilref = 0;
+            unsigned int stencilref = 0;
             int sense = 1;
             unsigned int changes = 0;
             std::vector<Batch>::const_iterator i = begin;
@@ -242,10 +414,11 @@ namespace OpenCSG {
                 glStencilFunc(GL_EQUAL, stencilref, OpenGL::stencilMask);
                 glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
 
-                { 
-                    for (Batch::const_iterator j = i->begin(); j != i->end(); ++j) {            
+                {
+                    for (Batch::const_iterator j = i->begin(); j != i->end(); ++j) {
                         RenderData* primitiveData = getRenderData(*j);
-                        GLubyte b = primitiveData->stencilID; glColor4ub(b, b, b, b);
+                        GLubyte * id = primitiveData->bufferId.vec();
+                        glColor4ubv(id);
                         (*j)->render();
                     }
                 }
@@ -275,7 +448,7 @@ namespace OpenCSG {
         bool subtractPrimitivesWithOcclusionQueries(std::vector<Batch>::const_iterator begin,
                                                     std::vector<Batch>::const_iterator end) {
 
-            const unsigned int numberOfBatches = end - begin;
+            const std::size_t numberOfBatches = end - begin;
             if (numberOfBatches == 0) {
                 return true;
             }
@@ -291,7 +464,7 @@ namespace OpenCSG {
 
             std::vector<unsigned int> fragmentcount(numberOfBatches, 0);
 
-            int stencilref = 0;
+            unsigned int stencilref = 0;
             std::vector<Batch>::const_iterator i = begin;
             unsigned int shapesWithoutUpdate = 0;    
             unsigned int shapeCount = 0;
@@ -329,10 +502,11 @@ namespace OpenCSG {
                 glStencilFunc(GL_EQUAL, stencilref, OpenGL::stencilMask);
                 glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
 
-                { 
-                    for (Batch::const_iterator j = i->begin(); j != i->end(); ++j) {            
+                {
+                    for (Batch::const_iterator j = i->begin(); j != i->end(); ++j) {
                         RenderData* primitiveData = getRenderData(*j);
-                        GLubyte b = primitiveData->stencilID; glColor4ub(b, b, b, b);
+                        GLubyte * id = primitiveData->bufferId.vec();
+                        glColor4ubv(id);
                         (*j)->render();
                     }
                 }
@@ -384,7 +558,7 @@ namespace OpenCSG {
 
     void renderSCS(const std::vector<Primitive*>& primitives, DepthComplexityAlgorithm algorithm) {
 
-        channelMgr = new SCSChannelManager;
+        channelMgr = getChannelManager();
         if (!channelMgr->init())
         {
             delete channelMgr;
@@ -403,7 +577,7 @@ namespace OpenCSG {
             for (std::vector<Primitive*>::const_iterator itr = primitives.begin(); itr != primitives.end(); ++itr) {
                 {
                     RenderData dta; 
-                    dta.stencilID = IDMaker.newID();
+                    dta.bufferId = IDMaker.newID();
                     gRenderInfo.insert(std::make_pair(*itr, dta));
                 }
                 Operation operation= (*itr)->getOperation();
@@ -411,7 +585,7 @@ namespace OpenCSG {
                     intersected.push_back(*itr);
                 } else if (operation == Subtraction) {
                     subtracted.push_back(*itr);
-                }   
+                }
             }
         }
 
@@ -426,12 +600,12 @@ namespace OpenCSG {
             glClear(GL_STENCIL_BUFFER_BIT);
             depthComplexity = 
                 (std::min)(OpenGL::calcMaxDepthComplexity(subtracted, scissor->getCurrentArea()), 
-                         subtractedBatches.size());
+                           static_cast<unsigned int>(subtractedBatches.size()));
         }
 
         channelMgr->request();
         channelMgr->renderToChannel(true);
-        
+
         scissor->enableScissor();
         scissor->store(channelMgr->current());
 
@@ -448,9 +622,10 @@ namespace OpenCSG {
         case OcclusionQuery:
             if (subtractPrimitivesWithOcclusionQueries(subtractedBatches.begin(), subtractedBatches.end()))
                 break; // success
-                       // else fall through (should we just give up here?)
+            // Maybe we just should give up here?
+            // fall through
         case NoDepthComplexitySampling:
-            subtractPrimitives(subtractedBatches.begin(), subtractedBatches.end(), subtractedBatches.size());
+            subtractPrimitives(subtractedBatches.begin(), subtractedBatches.end(), static_cast<unsigned int>(subtractedBatches.size()));
             break;
         case DepthComplexitySampling:
             subtractPrimitives(subtractedBatches.begin(), subtractedBatches.end(), depthComplexity);
